/*
 * Copyright 2010 James Pether Sörling
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *	$Id: RiksdagenApiImplTest.java 5466 2011-05-10 22:06:38Z pether $
 *  $HeadURL: https://cia.svn.sourceforge.net/svnroot/cia/tags/parent-pom-2011.05.31/service.external.riksdagen/src/test/java/com/hack23/cia/service/external/riksdagen/impl/RiksdagenApiImplTest.java $
*/
package com.hack23.cia.service.external.riksdagen.impl;

import java.util.ArrayList;
import java.util.List;

import org.junit.Test;
import org.springframework.beans.factory.annotation.Autowired;

import com.hack23.cia.model.external.riksdagen.documentcontent.impl.DocumentContentData;
import com.hack23.cia.model.external.riksdagen.dokumentlista.impl.DocumentElement;
import com.hack23.cia.model.external.riksdagen.dokumentstatus.impl.DocumentStatusContainer;
import com.hack23.cia.model.external.riksdagen.dokumentstatus.impl.DocumentType;
import com.hack23.cia.model.external.riksdagen.person.impl.PersonData;
import com.hack23.cia.model.external.riksdagen.personlista.impl.PersonContainerElement;
import com.hack23.cia.model.external.riksdagen.utskottsforslag.impl.CommitteeProposalComponentData;
import com.hack23.cia.model.external.riksdagen.votering.impl.VoteData;
import com.hack23.cia.model.external.riksdagen.voteringlista.impl.BallotDocumentElement;
import com.hack23.cia.service.external.common.api.ProcessDataStrategy;
import com.hack23.cia.service.external.riksdagen.api.RiksdagenApi;

/**
 * The Class RiksdagenApiImplTest.
 */
public class RiksdagenApiImplTest extends
		AbstractRiksdagenFunctionalIntegrationTest {

	/** The riksdagen api. */
	@Autowired
	RiksdagenApi riksdagenApi;

	/**
	 * Gets the ballot test.
	 *
	 * @return the ballot test
	 * @throws Exception
	 *             the exception
	 */
	@Test
	public void getBallotTest() throws Exception {
		final List<VoteData> votering = riksdagenApi
				.getBallot("E0B34EE1-3FD3-474A-81D7-781B04BB241E");
		assertNotNull(votering);
	}

	/**
	 * Gets the ballot list test.
	 *
	 * @return the ballot list test
	 * @throws Exception
	 *             the exception
	 */
	@Test
	public void getBallotListTest() throws Exception {
		final List<BallotDocumentElement> ballotList = riksdagenApi
				.getBallotList();
		assertNotNull(ballotList);

		assertTrue(ballotList.size() > 4000);

	}


	/**
	 * Gets the committee proposal test.
	 *
	 * @return the committee proposal test
	 * @throws Exception
	 *             the exception
	 */
	@Test
	public void getCommitteeProposalTest() throws Exception {
		final CommitteeProposalComponentData proposal = riksdagenApi
				.getCommitteeProposal("GX01FiU35");
		assertNotNull(proposal);
	}

	/**
	 * Gets the document content.
	 *
	 * @return the document content
	 * @throws Exception
	 *             the exception
	 */
	@Test
	public void getDocumentContent() throws Exception {
		final DocumentContentData documentContent = riksdagenApi
				.getDocumentContent("GX11916");
		assertNotNull(documentContent);
		assertTrue(documentContent.getContent().contains(
				"till statsrådet Cristina Husmark Pehrsson (m)"));
	}

	/**
	 * Gets the document list by changed since.
	 *
	 * @return the document list by changed since
	 * @throws Exception
	 *             the exception
	 */
	@Test
	public void getDocumentListByChangedSince() throws Exception {
		final List<DocumentElement> dokumentList = riksdagenApi.getDocumentList(
				"2010-06-01", 10);
		assertNotNull(dokumentList);
		assertTrue(dokumentList.size() >= 1);
	}



	/**
	 * Gets the document list by person.
	 *
	 * @return the document list by person
	 * @throws Exception
	 *             the exception
	 */
	@Test
	public void getDocumentListByPerson() throws Exception {
		final List<DocumentElement> dokumentList = riksdagenApi
				.getDocumentListByPerson("049290762318", 2010,10);
		assertNotNull(dokumentList);
		assertTrue(dokumentList.size() >= 1);
	}

	/**
	 * Gets the document list by type.
	 *
	 * @return the document list by type
	 * @throws Exception
	 *             the exception
	 */
	@Test
	public void getDocumentListByType() throws Exception {
		final List<DocumentElement> dokumentList = riksdagenApi.getDocumentList(
				DocumentType.BET, 10);
		assertNotNull(dokumentList);
		assertTrue(dokumentList.size() >= 100);
	}

	/**
	 * Gets the document list by type and year test.
	 *
	 * @return the document list by type and year test
	 * @throws Exception
	 *             the exception
	 */
	@Test
	public void getDocumentListByTypeAndYearTest() throws Exception {
		final List<DocumentElement> dokumentList = riksdagenApi.getDocumentList(
				DocumentType.BET, 2010, 10);
		assertNotNull(dokumentList);
		assertTrue(dokumentList.size() >= 100);
	}

	/**
	 * Gets the document list by year test.
	 *
	 * @return the document list by year test
	 * @throws Exception
	 *             the exception
	 */
	@Test
	public void getDocumentListByYearTest() throws Exception {
		final List<DocumentElement> dokumentList = riksdagenApi.getDocumentList(2010,
				10);
		assertNotNull(dokumentList);
		assertTrue(dokumentList.size() >= 100);
	}

	/**
	 * Gets the document status test.
	 *
	 * @return the document status test
	 * @throws Exception
	 *             the exception
	 */
	@Test
	public void getDocumentStatusTest() throws Exception {
		final DocumentStatusContainer dokumentstatus = riksdagenApi
				.getDocumentStatus("GU01JuU11");
		assertNotNull(dokumentstatus);
		assertNotNull(dokumentstatus.getDocument().getMadePublicDate());

	}

	// int votes = 0;
	// for (Dokument dokument : dokumentList) {
	// try {
	// DokumentstatusElement dokumentstatus = riksdagenApi
	// .getDocumentStatus(dokument.getId());
	//
	// assertNotNull(dokumentstatus);
	// System.out.println(dokumentstatus.getDokument().getStatus());
	// System.out.println(dokumentstatus.getDokument().getDokumentUrlText());
	//
	// System.out.println(dokument.getBeteckning());
	// System.out.println(dokument.getRm());
	// System.out.println(dokument.getUtskottsforslagUrlXml());
	//
	// String utskottsforslagUrlXml = dokument.getUtskottsforslagUrlXml();
	// if (utskottsforslagUrlXml != null) {
	// utskottsforslagUrlXml = utskottsforslagUrlXml.replaceAll(
	// "http://data.riksdagen.se/utskottsforslag/", "");
	//
	// try {
	// UtskottsforslagElement committeeProposal = riksdagenApi
	// .getCommitteeProposal(utskottsforslagUrlXml);
	// System.out
	// .println("committeeProposal.getDokutskottsforslag().getUtskottsforslag().size() "
	// + committeeProposal.getDokutskottsforslag()
	// .getUtskottsforslag().size());
	//
	// System.out.println(committeeProposal.getDokument()
	// .getBeteckning());
	// System.out.println(committeeProposal.getDokument()
	// .getPublicerad());
	// System.out.println(committeeProposal.getDokument()
	// .getDokumentstatusUrlXml());
	//
	// List<Utskottsforslag> utskottsforslagList = committeeProposal
	// .getDokutskottsforslag().getUtskottsforslag();
	//
	// for (Utskottsforslag utskottsforslag : utskottsforslagList) {
	// System.out.println("beslut typ : "
	// + utskottsforslag.getBeslutstyp());
	//
	// if (utskottsforslag.getVoteringUrlXml() != null
	// && (!"".equals(utskottsforslag
	// .getVoteringUrlXml()))) {
	// System.out.println("vote=" + votes + " .url="
	// + utskottsforslag.getVoteringUrlXml());
	//
	// try {
	// VoteringElement ballot = riksdagenApi
	// .getBallot(utskottsforslag
	// .getVoteringId());
	//
	// System.out.println(ballot.getDokvotering()
	// .getVotering().size());
	// System.out.println(ballot.getDokvotering()
	// .getVotering().get(0).getRm()
	// + " . "
	// + ballot.getDokvotering().getVotering()
	// .get(0).getBeteckning());
	// votes++;
	// } catch (Exception e) {
	// System.out.println("Problem : "
	// + utskottsforslag.getVoteringUrlXml());
	// }
	// }
	//
	// }
	//
	// } catch (Exception t) {
	// System.out.println("Problem with " + dokument.getUtskottsforslagUrlXml()
	// + t.getMessage());
	// }
	//
	// }
	// } catch (Exception e) {
	// System.out.println("Problem getting status : " + dokument.getId());
	// // TODO: handle exception
	// }
	// }

	/**
	 * Gets the person list test.
	 *
	 * @return the person list test
	 * @throws Exception
	 *             the exception
	 */
	@Test
	public void getPersonListTest() throws Exception {
		final PersonContainerElement personList = riksdagenApi.getPersonList();
		assertNotNull(personList);
		assertTrue(personList.getPerson().size() >= 1291);
	}

	/**
	 * Gets the person test.
	 *
	 * @return the person test
	 * @throws Exception
	 *             the exception
	 */
	@Test
	public void getPersonTest() throws Exception {
		final PersonData personElement = riksdagenApi.getPerson("0389619060499");
		assertNotNull(personElement);
		assertEquals("Bohman", personElement.getLastName());
	}

	/**
	 * Process document list by changed since.
	 *
	 * @throws Exception
	 *             the exception
	 */
	@Test
	public void processDocumentListByChangedSince() throws Exception {
		final List<DocumentElement> dokumentList = new ArrayList<DocumentElement>();

		final ProcessDataStrategy<DocumentElement> strategy = new ProcessDataStrategy<DocumentElement>() {
			@Override
			public void process(final DocumentElement t) {
				dokumentList.add(t);
			}
		};

		riksdagenApi.processDocumentList("2010-06-01", strategy);
		assertNotNull(dokumentList);
		assertTrue(dokumentList.size() >= 1);
	}

}
